package	jp.co.sds_corp.mail;

import	java.io.*;
import	java.util.*;
import	java.text.*;
import	java.rmi.*;

import	jp.co.sds_corp.util.common.*;
import	jp.co.sds_corp.util.config.*;
import	jp.co.sds_corp.util.daemon.*;

public class SmtpAuth extends AbstractService
{
	String	prop;
	File	file;
	BufferedReader	reader;
	boolean	bChange = false;
	String	fileName;
	String	frmLog;
	String	frmHost1;
	String	frmHost2;
	String	ipDir;
	String	setupDir;
	String	makeCmd;
	String	language;
	String	country;
	int		nPort;
	long	lLimit;
	long	lLen = -1;
	SimpleDateFormat	sdf = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss" );
	MessageFormat	fmtLine, fmtHost1, fmtHost2;

	public void init( IConfiguration config ) throws Exception {
		CmnLog.info( this, "LogAuth starting." );
		prop = config.get( "properties" );
		ResourceBundle res = PropertyResourceBundle.getBundle( prop );
		fileName = res.getString( "logFile" );
		frmLog = res.getString( "frmLog" );
		frmHost1 = res.getString( "frmHost1" );
		frmHost2 = res.getString( "frmHost2" );
		ipDir = res.getString( "ipDir" );
		setupDir = res.getString( "setupDir" );
		makeCmd = res.getString( "makeCmd" );
		lLimit = Long.valueOf( res.getString( "limitMinitues" )).longValue();
		language = res.getString( "language" );
		country = res.getString( "country" );
		Locale.setDefault( new Locale( language, country ));

		fmtLine = new MessageFormat( frmLog );
		fmtHost1 = new MessageFormat( frmHost1 );
		fmtHost2 = new MessageFormat( frmHost2 );

		file = new File( fileName );
		reader = openReader( file );
//		reader.skip( file.length() );
		endFile();

		CmnLog.info( this, "LogAuth started." );
	}

	public void start() {
		try {
			bChange = false;
			String line;
			for ( ;; ) {
				line = reader.readLine();
				if ( line == null ) {
					checkIp();
					if ( bChange ) {
						updateAccess();
					}
					bChange = false;
					if ( file.length() < lLen ) {
						reader = openReader( file );
					}
					lLen = file.length();
					break;
				} else {
					parseLine( line );
				}
			}
		} catch ( Exception e ) {
			CmnLog.error( this, "start error", e );
		}
	}

	BufferedReader openReader( File file ) throws FileNotFoundException {
		CmnLog.debug( this, "open file " + file );
		BufferedReader	reader = null;
		int		nRetry = 0;
		for ( ;; ) {
			try {
				reader = new BufferedReader( new FileReader( file ));
				break;
			} catch ( FileNotFoundException e ) {
				nRetry++;
				if ( nRetry > 5 ) {
					throw( e );
				}
				try {
					Thread.sleep( 1000 );
				} catch ( Exception ee ) {
				}
			}
		}
		return reader;
	}

	void endFile() {
		try {
			String line;
			for ( ;; ) {
				line = reader.readLine();
				if ( line == null ) {
					break;
				}
			}
		} catch ( Exception e ) {
			CmnLog.error( this, "end file error", e );
		}
	}

	private void parseLine( String line ) {
		// CmnLog.debug( this, "parse[" + frmLog + ":" + line + "]" );
		Object[]	res = null;
		try {
			res = fmtLine.parse( line );
		} catch ( ParseException e ) {
			CmnLog.error( this, "parse error:" + line + ":" + e.getMessage() );
			return;
		}
		String prc = (String )res[3];
		if ( !prc.startsWith( "ipop3d" )) {
			// CmnLog.debug( this, "not ipop3d:" + prc );
			return;
		}
		String cmd = (String )res[4];
		if ( !cmd.startsWith( "Login" ) && !cmd.startsWith( "Auth" )) {
			// CmnLog.debug( this, "not Login:" + cmd );
			return;
		}
		String ip = parseHost( (String )res[4] );
		if ( ip != null ) {
			addIp( ip );
		}
	}

	private String parseHost( String msg ) {
		Object[]	res = null;
		String ret = null;
		try {
			res = fmtHost1.parse( msg );
			ret = (String )res[3];
		} catch ( ParseException e ) {
			try {
				res = fmtHost2.parse( msg );
				ret = (String )res[2];
			} catch ( ParseException e2 ) {
				CmnLog.error( this, "parse host error:" + msg );
				return null;
			}
		}
		return ret;
	}

	private void addIp( String ip ) {
		CmnLog.info( this, "add:" + ip + ":" + sdf.format( new Date()) );
		bChange = true;
		try {
			FileOutputStream fo = new FileOutputStream( ipDir + "/" + ip );
			fo.close();
			fo = null;
		} catch ( Exception e ) {
			CmnLog.error( this, "add ip error", e );
		}
	}

	private void checkIp() {
		String[] files = new File( ipDir ).list();
		File	fl;
		long	mod, now;
		for ( int i = 0; i < files.length; i++ ) {
			fl = new File( ipDir + "/" + files[i] );
			mod = fl.lastModified();
			now = System.currentTimeMillis();
			if (( now - mod ) > ( lLimit * 60 * 1000 )) {
				bChange = true;
				fl.delete();
				CmnLog.info( this, "del:" + fl + ":" + sdf.format( new Date()) );
			}
		}
	}

	private void updateAccess() {
		File temp = null;
		try {
			String inFile = setupDir + "/access";

			String cmd;
			if ( makeCmd == null || makeCmd.length() == 0 ) {
				return;
			}
/*
			cmd = makeCmd + " -v hash " + setupDir + "/access.db";
			Process proc = Runtime.getRuntime().exec( cmd ) ;
			OutputStream os = proc.getOutputStream();
*/
			temp = File.createTempFile( "smtpauth", ".tmp" );
			OutputStream os = new FileOutputStream( temp );
			cmd = makeCmd + " " + temp;

			BufferedReader in = new BufferedReader( new FileReader( inFile ));
			PrintWriter out = new PrintWriter( new OutputStreamWriter( os ));
			String	line;
			for ( ;; ) {
				line = in.readLine();
				if ( line == null ) {
					break;
				}
				out.println( line );
			}
			in.close();

			String[] files = new File( ipDir ).list();
			File	fl;
			long	mod, now;
			for ( int i = 0; i < files.length; i++ ) {
				out.println( files[i] + "\tRELAY" );
			}
			out.flush();
			out.close();
			os.close();

			Process proc = Runtime.getRuntime().exec( cmd ) ;
/*
			Thread.sleep( 1000 );
			proc.destroy();
*/
			int nExit = proc.waitFor();
			if ( nExit == 0 ) {
				temp.delete();
			} else {
				CmnLog.error( this, cmd + " exit=" + nExit );
			}

		} catch ( Exception e ) {
			CmnLog.error( this, "update access error", e );
		} finally {
			if ( temp != null ) {
				//temp.delete();
			}
		}
	}

	public void stop() {
	}

	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "properties", prop );
	}
}
